@@ -25,9 +25,6 @@ namespace Aquamarine {
         virtual ~CDRMBufferAttachment() {
             ;
         }
-        virtual eAttachmentType type() {
-            return AQ_ATTACHMENT_DRM_BUFFER;
-        }
 
         Hyprutils::Memory::CSharedPointer<CDRMFB> fb;
     };
@@ -40,9 +37,6 @@ namespace Aquamarine {
         virtual ~CDRMBufferUnimportable() {
             ;
         }
-        virtual eAttachmentType type() {
-            return AQ_ATTACHMENT_DRM_KMS_UNIMPORTABLE;
-        }
     };
 
     class CDRMLease {
@@ -59,7 +53,7 @@ namespace Aquamarine {
         bool                                                     active = true;
 
         struct {
-            Hyprutils::Signal::CSignal destroy;
+            Hyprutils::Signal::CSignalT<> destroy;
         } events;
 
       private:
@@ -145,7 +139,7 @@ namespace Aquamarine {
                 uint32_t hotspot_x;
                 uint32_t hotspot_y;
                 uint32_t in_fence_fd;
-            };
+            } values;
             uint32_t props[17] = {0};
         };
         UDRMPlaneProps props;
@@ -179,14 +173,16 @@ namespace Aquamarine {
                 uint32_t gamma_lut;
                 uint32_t gamma_lut_size;
                 uint32_t ctm;
+                uint32_t degamma_lut;
+                uint32_t degamma_lut_size;
 
                 // atomic-modesetting only
 
                 uint32_t active;
                 uint32_t mode_id;
                 uint32_t out_fence_ptr;
-            };
-            uint32_t props[7] = {0};
+            } values;
+            uint32_t props[9] = {0};
         };
         UDRMCRTCProps props;
     };
@@ -203,6 +199,7 @@ namespace Aquamarine {
         virtual void                                                      setCursorVisible(bool visible);
         virtual Hyprutils::Math::Vector2D                                 cursorPlaneSize();
         virtual size_t                                                    getGammaSize();
+        virtual size_t                                                    getDeGammaSize();
         virtual std::vector<SDRMFormat>                                   getRenderFormats();
 
         int                                                               getConnectorID();
@@ -247,15 +244,20 @@ namespace Aquamarine {
         bool                                      test     = false;
         drmModeModeInfo                           modeInfo;
         std::optional<Hyprutils::Math::Mat3x3>    ctm;
+        std::optional<hdr_output_metadata>        hdrMetadata;
 
         struct {
-            uint32_t gammaLut = 0;
-            uint32_t fbDamage = 0;
-            uint32_t modeBlob = 0;
-            uint32_t ctmBlob  = 0;
-            bool     blobbed  = false;
-            bool     gammad   = false;
-            bool     ctmd     = false;
+            uint32_t gammaLut   = 0;
+            uint32_t degammaLut = 0;
+            uint32_t fbDamage   = 0;
+            uint32_t modeBlob   = 0;
+            uint32_t ctmBlob    = 0;
+            uint32_t hdrBlob    = 0;
+            bool     blobbed    = false;
+            bool     gammad     = false;
+            bool     degammad   = false;
+            bool     ctmd       = false;
+            bool     hdrd       = false; // true if hdr blob needs updating or clearing
         } atomic;
 
         void calculateMode(Hyprutils::Memory::CSharedPointer<SDRMConnector> connector);
@@ -269,7 +271,7 @@ namespace Aquamarine {
         void                                           disconnect();
         Hyprutils::Memory::CSharedPointer<SDRMCRTC>    getCurrentCRTC(const drmModeConnector* connector);
         drmModeModeInfo*                               getCurrentMode();
-        void                                           parseEDID(std::vector<uint8_t> data);
+        IOutput::SParsedEDID                           parseEDID(std::vector<uint8_t> data);
         bool                                           commitState(SDRMConnectorCommitData& data);
         void                                           applyCommit(const SDRMConnectorCommitData& data);
         void                                           rollbackCommit(const SDRMConnectorCommitData& data);
@@ -315,17 +317,29 @@ namespace Aquamarine {
                 uint32_t vrr_capable;  // not guaranteed to exist
                 uint32_t subconnector; // not guaranteed to exist
                 uint32_t non_desktop;
-                uint32_t panel_orientation; // not guaranteed to exist
-                uint32_t content_type;      // not guaranteed to exist
-                uint32_t max_bpc;           // not guaranteed to exist
+                uint32_t panel_orientation;   // not guaranteed to exist
+                uint32_t content_type;        // not guaranteed to exist
+                uint32_t max_bpc;             // not guaranteed to exist
+                uint32_t Colorspace;          // not guaranteed to exist
+                uint32_t hdr_output_metadata; // not guaranteed to exist
 
                 // atomic-modesetting only
 
                 uint32_t crtc_id;
-            };
-            uint32_t props[4] = {0};
+            } values;
+            uint32_t props[13] = {0};
         };
         UDRMConnectorProps props;
+
+        union UDRMConnectorColorspace {
+            struct {
+                uint32_t Default;
+                uint32_t BT2020_RGB;
+                uint32_t BT2020_YCC;
+            } values;
+            uint32_t props[3] = {0};
+        };
+        UDRMConnectorColorspace colorspace;
     };
 
     class IDRMImplementation {
@@ -355,6 +369,7 @@ namespace Aquamarine {
         virtual Hyprutils::Memory::CSharedPointer<IAllocator>              preferredAllocator();
         virtual std::vector<SDRMFormat>                                    getRenderableFormats();
         virtual std::vector<Hyprutils::Memory::CSharedPointer<IAllocator>> getAllocators();
+        virtual Hyprutils::Memory::CWeakPointer<IBackendImplementation>    getPrimary();
 
         Hyprutils::Memory::CWeakPointer<CDRMBackend>                       self;
 
@@ -369,15 +384,19 @@ namespace Aquamarine {
         CDRMBackend(Hyprutils::Memory::CSharedPointer<CBackend> backend);
 
         static std::vector<Hyprutils::Memory::CSharedPointer<CDRMBackend>> attempt(Hyprutils::Memory::CSharedPointer<CBackend> backend);
+        static Hyprutils::Memory::CSharedPointer<CDRMBackend>              fromGpu(std::string path, Hyprutils::Memory::CSharedPointer<CBackend> backend,
+                                                                                   Hyprutils::Memory::CSharedPointer<CDRMBackend> primary);
+
         bool registerGPU(Hyprutils::Memory::CSharedPointer<CSessionDevice> gpu_, Hyprutils::Memory::CSharedPointer<CDRMBackend> primary_ = {});
         bool checkFeatures();
         bool initResources();
         bool initMgpu();
         bool grabFormats();
         bool shouldBlit();
-        void scanConnectors(bool allowConnect = true);
+        void scanConnectors();
         void scanLeases();
         void restoreAfterVT();
+        void recheckOutputs();
         void recheckCRTCs();
         void buildGlFormats(const std::vector<SGLFormat>& fmts);
 
